 BSPEC 80
 TETRA 4 % Decent section length name (in 32-bit words).  However...
 ESPEC % Everything ends here.  The next thing is a LOP_LOC for .data, or
       % an ending LOP-something, hence a non-LOP_QUOTE in the name.

 .data
 TETRA 0x112233
