#as: --alternate
#objdump: -s -j .data
#name: alternate macro syntax

# Test the alternate macro syntax.

.*: .*

Contents of section .data:
 0000 01020912 61626331 32332121 3c3e2721 .*
 0010 3c3e27.*
