# objdump: -dr
# as: -linkrelax
# source: pop-op.s
.*:     file format elf64-mmix

Disassembly of section \.text:

0000000000000000 <Main>:
   0:	f8840def 	pop 132,3567
   4:	f8845678 	pop 132,22136
   8:	f8170def 	pop 23,3567
   c:	f8175678 	pop 23,22136
