.text
	.type _start,@function
_start:

	fma f4 = f5, f6, f7
	fma.s0 f4 = f5, f6, f7
	fma.s1 f4 = f5, f6, f7
	fma.s2 f4 = f5, f6, f7
	fma.s3 f4 = f5, f6, f7

	fma.s f4 = f5, f6, f7
	fma.s.s0 f4 = f5, f6, f7
	fma.s.s1 f4 = f5, f6, f7
	fma.s.s2 f4 = f5, f6, f7
	fma.s.s3 f4 = f5, f6, f7

	fma.d f4 = f5, f6, f7
	fma.d.s0 f4 = f5, f6, f7
	fma.d.s1 f4 = f5, f6, f7
	fma.d.s2 f4 = f5, f6, f7
	fma.d.s3 f4 = f5, f6, f7

	fpma f4 = f5, f6, f7
	fpma.s0 f4 = f5, f6, f7
	fpma.s1 f4 = f5, f6, f7
	fpma.s2 f4 = f5, f6, f7
	fpma.s3 f4 = f5, f6, f7

	fms f4 = f5, f6, f7
	fms.s0 f4 = f5, f6, f7
	fms.s1 f4 = f5, f6, f7
	fms.s2 f4 = f5, f6, f7
	fms.s3 f4 = f5, f6, f7

	fms.s f4 = f5, f6, f7
	fms.s.s0 f4 = f5, f6, f7
	fms.s.s1 f4 = f5, f6, f7
	fms.s.s2 f4 = f5, f6, f7
	fms.s.s3 f4 = f5, f6, f7

	fms.d f4 = f5, f6, f7
	fms.d.s0 f4 = f5, f6, f7
	fms.d.s1 f4 = f5, f6, f7
	fms.d.s2 f4 = f5, f6, f7
	fms.d.s3 f4 = f5, f6, f7

	fpms f4 = f5, f6, f7
	fpms.s0 f4 = f5, f6, f7
	fpms.s1 f4 = f5, f6, f7
	fpms.s2 f4 = f5, f6, f7
	fpms.s3 f4 = f5, f6, f7

	fnma f4 = f5, f6, f7
	fnma.s0 f4 = f5, f6, f7
	fnma.s1 f4 = f5, f6, f7
	fnma.s2 f4 = f5, f6, f7
	fnma.s3 f4 = f5, f6, f7

	fnma.s f4 = f5, f6, f7
	fnma.s.s0 f4 = f5, f6, f7
	fnma.s.s1 f4 = f5, f6, f7
	fnma.s.s2 f4 = f5, f6, f7
	fnma.s.s3 f4 = f5, f6, f7

	fnma.d f4 = f5, f6, f7
	fnma.d.s0 f4 = f5, f6, f7
	fnma.d.s1 f4 = f5, f6, f7
	fnma.d.s2 f4 = f5, f6, f7
	fnma.d.s3 f4 = f5, f6, f7

	fpnma f4 = f5, f6, f7
	fpnma.s0 f4 = f5, f6, f7
	fpnma.s1 f4 = f5, f6, f7
	fpnma.s2 f4 = f5, f6, f7
	fpnma.s3 f4 = f5, f6, f7

	fmpy f4 = f5, f6
	fmpy.s0 f4 = f5, f6
	fmpy.s1 f4 = f5, f6
	fmpy.s2 f4 = f5, f6
	fmpy.s3 f4 = f5, f6

	fmpy.s f4 = f5, f6
	fmpy.s.s0 f4 = f5, f6
	fmpy.s.s1 f4 = f5, f6
	fmpy.s.s2 f4 = f5, f6
	fmpy.s.s3 f4 = f5, f6

	fmpy.d f4 = f5, f6
	fmpy.d.s0 f4 = f5, f6
	fmpy.d.s1 f4 = f5, f6
	fmpy.d.s2 f4 = f5, f6
	fmpy.d.s3 f4 = f5, f6

	fpmpy f4 = f5, f6
	fpmpy.s0 f4 = f5, f6
	fpmpy.s1 f4 = f5, f6
	fpmpy.s2 f4 = f5, f6
	fpmpy.s3 f4 = f5, f6

	fadd f4 = f5, f6
	fadd.s0 f4 = f5, f6
	fadd.s1 f4 = f5, f6
	fadd.s2 f4 = f5, f6
	fadd.s3 f4 = f5, f6

	fadd.s f4 = f5, f6
	fadd.s.s0 f4 = f5, f6
	fadd.s.s1 f4 = f5, f6
	fadd.s.s2 f4 = f5, f6
	fadd.s.s3 f4 = f5, f6

	fadd.d f4 = f5, f6
	fadd.d.s0 f4 = f5, f6
	fadd.d.s1 f4 = f5, f6
	fadd.d.s2 f4 = f5, f6
	fadd.d.s3 f4 = f5, f6

	fsub f4 = f5, f6
	fsub.s0 f4 = f5, f6
	fsub.s1 f4 = f5, f6
	fsub.s2 f4 = f5, f6
	fsub.s3 f4 = f5, f6

	fsub.s f4 = f5, f6
	fsub.s.s0 f4 = f5, f6
	fsub.s.s1 f4 = f5, f6
	fsub.s.s2 f4 = f5, f6
	fsub.s.s3 f4 = f5, f6

	fsub.d f4 = f5, f6
	fsub.d.s0 f4 = f5, f6
	fsub.d.s1 f4 = f5, f6
	fsub.d.s2 f4 = f5, f6
	fsub.d.s3 f4 = f5, f6

	fnmpy f4 = f5, f6
	fnmpy.s0 f4 = f5, f6
	fnmpy.s1 f4 = f5, f6
	fnmpy.s2 f4 = f5, f6
	fnmpy.s3 f4 = f5, f6

	fnmpy.s f4 = f5, f6
	fnmpy.s.s0 f4 = f5, f6
	fnmpy.s.s1 f4 = f5, f6
	fnmpy.s.s2 f4 = f5, f6
	fnmpy.s.s3 f4 = f5, f6

	fnmpy.d f4 = f5, f6
	fnmpy.d.s0 f4 = f5, f6
	fnmpy.d.s1 f4 = f5, f6
	fnmpy.d.s2 f4 = f5, f6
	fnmpy.d.s3 f4 = f5, f6

	fpnmpy f4 = f5, f6
	fpnmpy.s0 f4 = f5, f6
	fpnmpy.s1 f4 = f5, f6
	fpnmpy.s2 f4 = f5, f6
	fpnmpy.s3 f4 = f5, f6

	fnorm f4 = f5
	fnorm.s0 f4 = f5
	fnorm.s1 f4 = f5
	fnorm.s2 f4 = f5
	fnorm.s3 f4 = f5

	fnorm.s f4 = f5
	fnorm.s.s0 f4 = f5
	fnorm.s.s1 f4 = f5
	fnorm.s.s2 f4 = f5
	fnorm.s.s3 f4 = f5

	fnorm.d f4 = f5
	fnorm.d.s0 f4 = f5
	fnorm.d.s1 f4 = f5
	fnorm.d.s2 f4 = f5
	fnorm.d.s3 f4 = f5

	xma.l f4 = f5, f6, f7
	xma.lu f4 = f5, f6, f7
	xma.h f4 = f5, f6, f7
	xma.hu f4 = f5, f6, f7

	xmpy.l f4 = f5, f6
	xmpy.lu f4 = f5, f6
	xmpy.h f4 = f5, f6
	xmpy.hu f4 = f5, f6

	fselect f4 = f5, f6, f7

	fcmp.eq p3, p4 = f4, f5
	fcmp.eq.s0 p3, p4 = f4, f5
	fcmp.eq.s1 p3, p4 = f4, f5
	fcmp.eq.s2 p3, p4 = f4, f5
	fcmp.eq.s3 p3, p4 = f4, f5
	fcmp.eq.unc p3, p4 = f4, f5
	fcmp.eq.unc.s0 p3, p4 = f4, f5
	fcmp.eq.unc.s1 p3, p4 = f4, f5
	fcmp.eq.unc.s2 p3, p4 = f4, f5
	fcmp.eq.unc.s3 p3, p4 = f4, f5

	fcmp.lt p3, p4 = f4, f5
	fcmp.lt.s0 p3, p4 = f4, f5
	fcmp.lt.s1 p3, p4 = f4, f5
	fcmp.lt.s2 p3, p4 = f4, f5
	fcmp.lt.s3 p3, p4 = f4, f5
	fcmp.lt.unc p3, p4 = f4, f5
	fcmp.lt.unc.s0 p3, p4 = f4, f5
	fcmp.lt.unc.s1 p3, p4 = f4, f5
	fcmp.lt.unc.s2 p3, p4 = f4, f5
	fcmp.lt.unc.s3 p3, p4 = f4, f5

	fcmp.le p3, p4 = f4, f5
	fcmp.le.s0 p3, p4 = f4, f5
	fcmp.le.s1 p3, p4 = f4, f5
	fcmp.le.s2 p3, p4 = f4, f5
	fcmp.le.s3 p3, p4 = f4, f5
	fcmp.le.unc p3, p4 = f4, f5
	fcmp.le.unc.s0 p3, p4 = f4, f5
	fcmp.le.unc.s1 p3, p4 = f4, f5
	fcmp.le.unc.s2 p3, p4 = f4, f5
	fcmp.le.unc.s3 p3, p4 = f4, f5

	fcmp.unord p3, p4 = f4, f5
	fcmp.unord.s0 p3, p4 = f4, f5
	fcmp.unord.s1 p3, p4 = f4, f5
	fcmp.unord.s2 p3, p4 = f4, f5
	fcmp.unord.s3 p3, p4 = f4, f5
	fcmp.unord.unc p3, p4 = f4, f5
	fcmp.unord.unc.s0 p3, p4 = f4, f5
	fcmp.unord.unc.s1 p3, p4 = f4, f5
	fcmp.unord.unc.s2 p3, p4 = f4, f5
	fcmp.unord.unc.s3 p3, p4 = f4, f5

	fcmp.gt p3, p4 = f4, f5
	fcmp.gt.s0 p3, p4 = f4, f5
	fcmp.gt.s1 p3, p4 = f4, f5
	fcmp.gt.s2 p3, p4 = f4, f5
	fcmp.gt.s3 p3, p4 = f4, f5
	fcmp.gt.unc p3, p4 = f4, f5
	fcmp.gt.unc.s0 p3, p4 = f4, f5
	fcmp.gt.unc.s1 p3, p4 = f4, f5
	fcmp.gt.unc.s2 p3, p4 = f4, f5
	fcmp.gt.unc.s3 p3, p4 = f4, f5

	fcmp.ge p3, p4 = f4, f5
	fcmp.ge.s0 p3, p4 = f4, f5
	fcmp.ge.s1 p3, p4 = f4, f5
	fcmp.ge.s2 p3, p4 = f4, f5
	fcmp.ge.s3 p3, p4 = f4, f5
	fcmp.ge.unc p3, p4 = f4, f5
	fcmp.ge.unc.s0 p3, p4 = f4, f5
	fcmp.ge.unc.s1 p3, p4 = f4, f5
	fcmp.ge.unc.s2 p3, p4 = f4, f5
	fcmp.ge.unc.s3 p3, p4 = f4, f5

	fcmp.neq p3, p4 = f4, f5
	fcmp.neq.s0 p3, p4 = f4, f5
	fcmp.neq.s1 p3, p4 = f4, f5
	fcmp.neq.s2 p3, p4 = f4, f5
	fcmp.neq.s3 p3, p4 = f4, f5
	fcmp.neq.unc p3, p4 = f4, f5
	fcmp.neq.unc.s0 p3, p4 = f4, f5
	fcmp.neq.unc.s1 p3, p4 = f4, f5
	fcmp.neq.unc.s2 p3, p4 = f4, f5
	fcmp.neq.unc.s3 p3, p4 = f4, f5

	fcmp.nlt p3, p4 = f4, f5
	fcmp.nlt.s0 p3, p4 = f4, f5
	fcmp.nlt.s1 p3, p4 = f4, f5
	fcmp.nlt.s2 p3, p4 = f4, f5
	fcmp.nlt.s3 p3, p4 = f4, f5
	fcmp.nlt.unc p3, p4 = f4, f5
	fcmp.nlt.unc.s0 p3, p4 = f4, f5
	fcmp.nlt.unc.s1 p3, p4 = f4, f5
	fcmp.nlt.unc.s2 p3, p4 = f4, f5
	fcmp.nlt.unc.s3 p3, p4 = f4, f5

	fcmp.nle p3, p4 = f4, f5
	fcmp.nle.s0 p3, p4 = f4, f5
	fcmp.nle.s1 p3, p4 = f4, f5
	fcmp.nle.s2 p3, p4 = f4, f5
	fcmp.nle.s3 p3, p4 = f4, f5
	fcmp.nle.unc p3, p4 = f4, f5
	fcmp.nle.unc.s0 p3, p4 = f4, f5
	fcmp.nle.unc.s1 p3, p4 = f4, f5
	fcmp.nle.unc.s2 p3, p4 = f4, f5
	fcmp.nle.unc.s3 p3, p4 = f4, f5

	fcmp.ngt p3, p4 = f4, f5
	fcmp.ngt.s0 p3, p4 = f4, f5
	fcmp.ngt.s1 p3, p4 = f4, f5
	fcmp.ngt.s2 p3, p4 = f4, f5
	fcmp.ngt.s3 p3, p4 = f4, f5
	fcmp.ngt.unc p3, p4 = f4, f5
	fcmp.ngt.unc.s0 p3, p4 = f4, f5
	fcmp.ngt.unc.s1 p3, p4 = f4, f5
	fcmp.ngt.unc.s2 p3, p4 = f4, f5
	fcmp.ngt.unc.s3 p3, p4 = f4, f5

	fcmp.nge p3, p4 = f4, f5
	fcmp.nge.s0 p3, p4 = f4, f5
	fcmp.nge.s1 p3, p4 = f4, f5
	fcmp.nge.s2 p3, p4 = f4, f5
	fcmp.nge.s3 p3, p4 = f4, f5
	fcmp.nge.unc p3, p4 = f4, f5
	fcmp.nge.unc.s0 p3, p4 = f4, f5
	fcmp.nge.unc.s1 p3, p4 = f4, f5
	fcmp.nge.unc.s2 p3, p4 = f4, f5
	fcmp.nge.unc.s3 p3, p4 = f4, f5

	fcmp.ord p3, p4 = f4, f5
	fcmp.ord.s0 p3, p4 = f4, f5
	fcmp.ord.s1 p3, p4 = f4, f5
	fcmp.ord.s2 p3, p4 = f4, f5
	fcmp.ord.s3 p3, p4 = f4, f5
	fcmp.ord.unc p3, p4 = f4, f5
	fcmp.ord.unc.s0 p3, p4 = f4, f5
	fcmp.ord.unc.s1 p3, p4 = f4, f5
	fcmp.ord.unc.s2 p3, p4 = f4, f5
	fcmp.ord.unc.s3 p3, p4 = f4, f5

	fclass.m p3, p4 = f4, @nat
	fclass.nm p3, p4 = f4, @nat
	fclass.m p3, p4 = f4, @qnan
	fclass.nm p3, p4 = f4, @qnan
	fclass.m p3, p4 = f4, @snan
	fclass.nm p3, p4 = f4, @snan
	fclass.m p3, p4 = f4, @pos
	fclass.nm p3, p4 = f4, @pos
	fclass.m p3, p4 = f4, @neg
	fclass.nm p3, p4 = f4, @neg
	fclass.m p3, p4 = f4, @unorm
	fclass.nm p3, p4 = f4, @unorm
	fclass.m p3, p4 = f4, @norm
	fclass.nm p3, p4 = f4, @norm
	fclass.m p3, p4 = f4, @inf
	fclass.nm p3, p4 = f4, @inf
	fclass.m p3, p4 = f4, 0x1ff
	fclass.nm p3, p4 = f4, 0x1ff

	fclass.m.unc p3, p4 = f4, @nat
	fclass.nm.unc p3, p4 = f4, @nat
	fclass.m.unc p3, p4 = f4, @qnan
	fclass.nm.unc p3, p4 = f4, @qnan
	fclass.m.unc p3, p4 = f4, @snan
	fclass.nm.unc p3, p4 = f4, @snan
	fclass.m.unc p3, p4 = f4, @pos
	fclass.nm.unc p3, p4 = f4, @pos
	fclass.m.unc p3, p4 = f4, @neg
	fclass.nm.unc p3, p4 = f4, @neg
	fclass.m.unc p3, p4 = f4, @unorm
	fclass.nm.unc p3, p4 = f4, @unorm
	fclass.m.unc p3, p4 = f4, @norm
	fclass.nm.unc p3, p4 = f4, @norm
	fclass.m.unc p3, p4 = f4, @inf
	fclass.nm.unc p3, p4 = f4, @inf
	fclass.m.unc p3, p4 = f4, 0x1ff
	fclass.nm.unc p3, p4 = f4, 0x1ff

	frcpa f4, p5 = f6, f7
	frcpa.s0 f4, p5 = f6, f7
	frcpa.s1 f4, p5 = f6, f7
	frcpa.s2 f4, p5 = f6, f7
	frcpa.s3 f4, p5 = f6, f7

	fprcpa f4, p5 = f6, f7
	fprcpa.s0 f4, p5 = f6, f7
	fprcpa.s1 f4, p5 = f6, f7
	fprcpa.s2 f4, p5 = f6, f7
	fprcpa.s3 f4, p5 = f6, f7

	frsqrta f4, p5 = f6
	frsqrta.s0 f4, p5 = f6
	frsqrta.s1 f4, p5 = f6
	frsqrta.s2 f4, p5 = f6
	frsqrta.s3 f4, p5 = f6

	fprsqrta f4, p5 = f6
	fprsqrta.s0 f4, p5 = f6
	fprsqrta.s1 f4, p5 = f6
	fprsqrta.s2 f4, p5 = f6
	fprsqrta.s3 f4, p5 = f6

	fmin f4 = f5, f6
	fmin.s0 f4 = f5, f6
	fmin.s1 f4 = f5, f6
	fmin.s2 f4 = f5, f6
	fmin.s3 f4 = f5, f6

	fmax f4 = f5, f6
	fmax.s0 f4 = f5, f6
	fmax.s1 f4 = f5, f6
	fmax.s2 f4 = f5, f6
	fmax.s3 f4 = f5, f6

	famin f4 = f5, f6
	famin.s0 f4 = f5, f6
	famin.s1 f4 = f5, f6
	famin.s2 f4 = f5, f6
	famin.s3 f4 = f5, f6

	famax f4 = f5, f6
	famax.s0 f4 = f5, f6
	famax.s1 f4 = f5, f6
	famax.s2 f4 = f5, f6
	famax.s3 f4 = f5, f6

	fpmin f4 = f5, f6
	fpmin.s0 f4 = f5, f6
	fpmin.s1 f4 = f5, f6
	fpmin.s2 f4 = f5, f6
	fpmin.s3 f4 = f5, f6

	fpmax f4 = f5, f6
	fpmax.s0 f4 = f5, f6
	fpmax.s1 f4 = f5, f6
	fpmax.s2 f4 = f5, f6
	fpmax.s3 f4 = f5, f6

	fpamin f4 = f5, f6
	fpamin.s0 f4 = f5, f6
	fpamin.s1 f4 = f5, f6
	fpamin.s2 f4 = f5, f6
	fpamin.s3 f4 = f5, f6

	fpamax f4 = f5, f6
	fpamax.s0 f4 = f5, f6
	fpamax.s1 f4 = f5, f6
	fpamax.s2 f4 = f5, f6
	fpamax.s3 f4 = f5, f6

	fpcmp.eq f3 = f4, f5
	fpcmp.eq.s0 f3 = f4, f5
	fpcmp.eq.s1 f3 = f4, f5
	fpcmp.eq.s2 f3 = f4, f5
	fpcmp.eq.s3 f3 = f4, f5

	fpcmp.lt f3 = f4, f5
	fpcmp.lt.s0 f3 = f4, f5
	fpcmp.lt.s1 f3 = f4, f5
	fpcmp.lt.s2 f3 = f4, f5
	fpcmp.lt.s3 f3 = f4, f5

	fpcmp.le f3 = f4, f5
	fpcmp.le.s0 f3 = f4, f5
	fpcmp.le.s1 f3 = f4, f5
	fpcmp.le.s2 f3 = f4, f5
	fpcmp.le.s3 f3 = f4, f5

	fpcmp.unord f3 = f4, f5
	fpcmp.unord.s0 f3 = f4, f5
	fpcmp.unord.s1 f3 = f4, f5
	fpcmp.unord.s2 f3 = f4, f5
	fpcmp.unord.s3 f3 = f4, f5

	fpcmp.gt f3 = f4, f5
	fpcmp.gt.s0 f3 = f4, f5
	fpcmp.gt.s1 f3 = f4, f5
	fpcmp.gt.s2 f3 = f4, f5
	fpcmp.gt.s3 f3 = f4, f5

	fpcmp.ge f3 = f4, f5
	fpcmp.ge.s0 f3 = f4, f5
	fpcmp.ge.s1 f3 = f4, f5
	fpcmp.ge.s2 f3 = f4, f5
	fpcmp.ge.s3 f3 = f4, f5

	fpcmp.neq f3 = f4, f5
	fpcmp.neq.s0 f3 = f4, f5
	fpcmp.neq.s1 f3 = f4, f5
	fpcmp.neq.s2 f3 = f4, f5
	fpcmp.neq.s3 f3 = f4, f5

	fpcmp.nlt f3 = f4, f5
	fpcmp.nlt.s0 f3 = f4, f5
	fpcmp.nlt.s1 f3 = f4, f5
	fpcmp.nlt.s2 f3 = f4, f5
	fpcmp.nlt.s3 f3 = f4, f5

	fpcmp.nle f3 = f4, f5
	fpcmp.nle.s0 f3 = f4, f5
	fpcmp.nle.s1 f3 = f4, f5
	fpcmp.nle.s2 f3 = f4, f5
	fpcmp.nle.s3 f3 = f4, f5

	fpcmp.ngt f3 = f4, f5
	fpcmp.ngt.s0 f3 = f4, f5
	fpcmp.ngt.s1 f3 = f4, f5
	fpcmp.ngt.s2 f3 = f4, f5
	fpcmp.ngt.s3 f3 = f4, f5

	fpcmp.nge f3 = f4, f5
	fpcmp.nge.s0 f3 = f4, f5
	fpcmp.nge.s1 f3 = f4, f5
	fpcmp.nge.s2 f3 = f4, f5
	fpcmp.nge.s3 f3 = f4, f5

	fpcmp.ord f3 = f4, f5
	fpcmp.ord.s0 f3 = f4, f5
	fpcmp.ord.s1 f3 = f4, f5
	fpcmp.ord.s2 f3 = f4, f5
	fpcmp.ord.s3 f3 = f4, f5

	fmerge.s f4 = f5, f6
	fmerge.ns f4 = f5, f6
	fmerge.se f4 = f5, f6
	fmix.lr f4 = f5, f6
	fmix.r f4 = f5, f6
	fmix.l f4 = f5, f6
	fsxt.l f4 = f5, f6
	fpack f4 = f5, f6
	fswap f4 = f5, f6
	fswap.nl f4 = f5, f6
	fswap.nr f4 = f5, f6
	fand f4 = f5, f6
	fandcm f4 = f5, f6
	for f4 = f5, f6
	fxor f4 = f5, f6
	fpmerge.s f4 = f5, f6
	fpmerge.ns f4 = f5, f6
	fpmerge.se f4 = f5, f6

	fabs f4 = f5
	fneg f4 = f5
	fnegabs f4 = f5
	fpabs f4 = f5
	fpneg f4 = f5
	fpnegabs f4 = f5

	fcvt.fx f4 = f5
	fcvt.fx.s0 f4 = f5
	fcvt.fx.s1 f4 = f5
	fcvt.fx.s2 f4 = f5
	fcvt.fx.s3 f4 = f5

	fcvt.fx.trunc f4 = f5
	fcvt.fx.trunc.s0 f4 = f5
	fcvt.fx.trunc.s1 f4 = f5
	fcvt.fx.trunc.s2 f4 = f5
	fcvt.fx.trunc.s3 f4 = f5

	fcvt.fxu f4 = f5
	fcvt.fxu.s0 f4 = f5
	fcvt.fxu.s1 f4 = f5
	fcvt.fxu.s2 f4 = f5
	fcvt.fxu.s3 f4 = f5

	fcvt.fxu.trunc f4 = f5
	fcvt.fxu.trunc.s0 f4 = f5
	fcvt.fxu.trunc.s1 f4 = f5
	fcvt.fxu.trunc.s2 f4 = f5
	fcvt.fxu.trunc.s3 f4 = f5

	fpcvt.fx f4 = f5
	fpcvt.fx.s0 f4 = f5
	fpcvt.fx.s1 f4 = f5
	fpcvt.fx.s2 f4 = f5
	fpcvt.fx.s3 f4 = f5

	fpcvt.fx.trunc f4 = f5
	fpcvt.fx.trunc.s0 f4 = f5
	fpcvt.fx.trunc.s1 f4 = f5
	fpcvt.fx.trunc.s2 f4 = f5
	fpcvt.fx.trunc.s3 f4 = f5

	fpcvt.fxu f4 = f5
	fpcvt.fxu.s0 f4 = f5
	fpcvt.fxu.s1 f4 = f5
	fpcvt.fxu.s2 f4 = f5
	fpcvt.fxu.s3 f4 = f5

	fpcvt.fxu.trunc f4 = f5
	fpcvt.fxu.trunc.s0 f4 = f5
	fpcvt.fxu.trunc.s1 f4 = f5
	fpcvt.fxu.trunc.s2 f4 = f5
	fpcvt.fxu.trunc.s3 f4 = f5

	fcvt.xf f4 = f5
	fcvt.xuf f4 = f5

	fsetc 0, 0
	fsetc 0x3f, 0x3f
	fsetc.s0 0, 0
	fsetc.s0 0x3f, 0x3f
	fsetc.s1 0, 0
	fsetc.s1 0x3f, 0x3f
	fsetc.s2 0, 0
	fsetc.s2 0x3f, 0x3f
	fsetc.s3 0, 0
	fsetc.s3 0x3f, 0x3f

	fclrf
	fclrf.s0
	fclrf.s1
	fclrf.s2
	fclrf.s3

	fchkf _start
	fchkf.s0 _start
	fchkf.s1 _start
	fchkf.s2 _start
	fchkf.s3 _start

	break.f 0
	nop.f 0;;

	# instructions added by SDM2.1:

	hint.f 0
	hint.f @pause
	hint.f 0x1ffff
