.*: Assembler messages:
.*:5: Warning: relaxed out-of-range branch into a jump
.*:6: Warning: relaxed out-of-range branch into a jump
.*:7: Warning: relaxed out-of-range branch into a jump
.*:8: Warning: relaxed out-of-range branch into a jump
.*:9: Warning: relaxed out-of-range branch into a jump
.*:10: Warning: relaxed out-of-range branch into a jump
.*:11: Warning: relaxed out-of-range branch into a jump
.*:12: Warning: relaxed out-of-range branch into a jump
.*:13: Warning: relaxed out-of-range branch into a jump
.*:14: Warning: relaxed out-of-range branch into a jump
.*:16: Warning: relaxed out-of-range branch into a jump
.*:17: Warning: relaxed out-of-range branch into a jump
.*:19: Warning: relaxed out-of-range branch into a jump
.*:20: Warning: relaxed out-of-range branch into a jump
.*:21: Warning: relaxed out-of-range branch into a jump
.*:22: Warning: relaxed out-of-range branch into a jump
.*:23: Warning: relaxed out-of-range branch into a jump
.*:24: Warning: relaxed out-of-range branch into a jump
.*:25: Warning: relaxed out-of-range branch into a jump
.*:26: Warning: relaxed out-of-range branch into a jump
.*:28: Warning: relaxed out-of-range branch into a jump
.*:29: Warning: relaxed out-of-range branch into a jump
.*:33: Warning: relaxed out-of-range branch into a jump
.*:34: Warning: relaxed out-of-range branch into a jump
.*:35: Warning: relaxed out-of-range branch into a jump
.*:36: Warning: relaxed out-of-range branch into a jump
.*:37: Warning: relaxed out-of-range branch into a jump
.*:38: Warning: relaxed out-of-range branch into a jump
.*:39: Warning: relaxed out-of-range branch into a jump
.*:40: Warning: relaxed out-of-range branch into a jump
.*:41: Warning: relaxed out-of-range branch into a jump
.*:42: Warning: relaxed out-of-range branch into a jump
.*:44: Warning: relaxed out-of-range branch into a jump
.*:45: Warning: relaxed out-of-range branch into a jump
.*:47: Warning: relaxed out-of-range branch into a jump
.*:48: Warning: relaxed out-of-range branch into a jump
.*:49: Warning: relaxed out-of-range branch into a jump
.*:50: Warning: relaxed out-of-range branch into a jump
.*:51: Warning: relaxed out-of-range branch into a jump
.*:52: Warning: relaxed out-of-range branch into a jump
.*:53: Warning: relaxed out-of-range branch into a jump
.*:54: Warning: relaxed out-of-range branch into a jump
.*:56: Warning: relaxed out-of-range branch into a jump
.*:57: Warning: relaxed out-of-range branch into a jump
