! Section with SHmedia in unique section, similar to crange-2f.s

	.section .text.2g,"ax"
	.align 2

	.mode SHmedia
	movi 0x21,r12
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movi 0x21,r13
