#objdump: -d
#name: alpha unop disassembly

.*:     file format .*

Disassembly of section .text:

0*0000000 <.text>:
   0:	00 00 fe 2f 	unop	
   4:	00 00 e0 2f 	unop	
   8:	00 00 fe 2f 	unop	
   c:	00 00 ff 2f 	unop	
