GAS for MMIX .*/1hjmp1b.s 			page 1


   1 0000 FD000000 	Main SWYM 0,0,0
   2 0004 F0000001 	1H JMP 1F
   3 0008 F1FFFFFF 	1H JMP 1B
   4 000c F1FFFFFF 	 JMP 1B
GAS for MMIX .*/1hjmp1b.s 			page 2


DEFINED SYMBOLS
.*/1hjmp1b.s:1      .text:0000000000000000 Main

NO UNDEFINED SYMBOLS
