[^:]*: Assembler messages:
[^:]*:36: Error: lo register required -- `tst r8,r0'
[^:]*:36: Error: lo register required -- `tst r0,r8'
[^:]*:36: Error: unshifted register required -- `tst r0,#12'
[^:]*:36: Error: unshifted register required -- `tst r0,r1,lsl#2'
[^:]*:36: Error: unshifted register required -- `tst r0,r1,lsl r3'
[^:]*:37: Error: lo register required -- `cmn r8,r0'
[^:]*:37: Error: lo register required -- `cmn r0,r8'
[^:]*:37: Error: unshifted register required -- `cmn r0,#12'
[^:]*:37: Error: unshifted register required -- `cmn r0,r1,lsl#2'
[^:]*:37: Error: unshifted register required -- `cmn r0,r1,lsl r3'
[^:]*:38: Error: lo register required -- `mvn r8,r0'
[^:]*:38: Error: lo register required -- `mvn r0,r8'
[^:]*:38: Error: unshifted register required -- `mvn r0,#12'
[^:]*:38: Error: unshifted register required -- `mvn r0,r1,lsl#2'
[^:]*:38: Error: unshifted register required -- `mvn r0,r1,lsl r3'
[^:]*:39: Error: lo register required -- `neg r8,r0'
[^:]*:39: Error: lo register required -- `neg r0,r8'
[^:]*:40: Error: lo register required -- `rev r8,r0'
[^:]*:40: Error: lo register required -- `rev r0,r8'
[^:]*:41: Error: lo register required -- `rev16 r8,r0'
[^:]*:41: Error: lo register required -- `rev16 r0,r8'
[^:]*:42: Error: lo register required -- `revsh r8,r0'
[^:]*:42: Error: lo register required -- `revsh r0,r8'
[^:]*:43: Error: lo register required -- `sxtb r8,r0'
[^:]*:43: Error: lo register required -- `sxtb r0,r8'
[^:]*:43: Error: Thumb encoding does not support rotation -- `sxtb r0,r1,ror#8'
[^:]*:44: Error: lo register required -- `sxth r8,r0'
[^:]*:44: Error: lo register required -- `sxth r0,r8'
[^:]*:44: Error: Thumb encoding does not support rotation -- `sxth r0,r1,ror#8'
[^:]*:45: Error: lo register required -- `uxtb r8,r0'
[^:]*:45: Error: lo register required -- `uxtb r0,r8'
[^:]*:45: Error: Thumb encoding does not support rotation -- `uxtb r0,r1,ror#8'
[^:]*:46: Error: lo register required -- `uxth r8,r0'
[^:]*:46: Error: lo register required -- `uxth r0,r8'
[^:]*:46: Error: Thumb encoding does not support rotation -- `uxth r0,r1,ror#8'
[^:]*:48: Error: dest must overlap one source register -- `adc r1,r2,r3'
[^:]*:48: Error: lo register required -- `adc r8,r0'
[^:]*:48: Error: lo register required -- `adc r0,r8'
[^:]*:48: Error: unshifted register required -- `adc r0,#12'
[^:]*:48: Error: unshifted register required -- `adc r0,r1,lsl#2'
[^:]*:48: Error: unshifted register required -- `adc r0,r1,lsl r3'
[^:]*:49: Error: dest must overlap one source register -- `and r1,r2,r3'
[^:]*:49: Error: lo register required -- `and r8,r0'
[^:]*:49: Error: lo register required -- `and r0,r8'
[^:]*:49: Error: unshifted register required -- `and r0,#12'
[^:]*:49: Error: unshifted register required -- `and r0,r1,lsl#2'
[^:]*:49: Error: unshifted register required -- `and r0,r1,lsl r3'
[^:]*:50: Error: dest and source1 must be the same register -- `bic r1,r2,r3'
[^:]*:50: Error: lo register required -- `bic r8,r0'
[^:]*:50: Error: lo register required -- `bic r0,r8'
[^:]*:50: Error: unshifted register required -- `bic r0,#12'
[^:]*:50: Error: unshifted register required -- `bic r0,r1,lsl#2'
[^:]*:50: Error: unshifted register required -- `bic r0,r1,lsl r3'
[^:]*:51: Error: dest must overlap one source register -- `eor r1,r2,r3'
[^:]*:51: Error: lo register required -- `eor r8,r0'
[^:]*:51: Error: lo register required -- `eor r0,r8'
[^:]*:51: Error: unshifted register required -- `eor r0,#12'
[^:]*:51: Error: unshifted register required -- `eor r0,r1,lsl#2'
[^:]*:51: Error: unshifted register required -- `eor r0,r1,lsl r3'
[^:]*:52: Error: dest must overlap one source register -- `orr r1,r2,r3'
[^:]*:52: Error: lo register required -- `orr r8,r0'
[^:]*:52: Error: lo register required -- `orr r0,r8'
[^:]*:52: Error: unshifted register required -- `orr r0,#12'
[^:]*:52: Error: unshifted register required -- `orr r0,r1,lsl#2'
[^:]*:52: Error: unshifted register required -- `orr r0,r1,lsl r3'
[^:]*:53: Error: dest and source1 must be the same register -- `sbc r1,r2,r3'
[^:]*:53: Error: lo register required -- `sbc r8,r0'
[^:]*:53: Error: lo register required -- `sbc r0,r8'
[^:]*:53: Error: unshifted register required -- `sbc r0,#12'
[^:]*:53: Error: unshifted register required -- `sbc r0,r1,lsl#2'
[^:]*:53: Error: unshifted register required -- `sbc r0,r1,lsl r3'
[^:]*:54: Error: dest must overlap one source register -- `mul r1,r2,r3'
[^:]*:54: Error: lo register required -- `mul r8,r0'
[^:]*:54: Error: lo register required -- `mul r0,r8'
[^:]*:62: Error: lo register required -- `asr r8,r0,#12'
[^:]*:62: Error: lo register required -- `asr r0,r8,#12'
[^:]*:62: Error: lo register required -- `asr r8,r0'
[^:]*:62: Error: lo register required -- `asr r0,r8'
[^:]*:63: Error: lo register required -- `lsl r8,r0,#12'
[^:]*:63: Error: lo register required -- `lsl r0,r8,#12'
[^:]*:63: Error: lo register required -- `lsl r8,r0'
[^:]*:63: Error: lo register required -- `lsl r0,r8'
[^:]*:64: Error: lo register required -- `lsr r8,r0,#12'
[^:]*:64: Error: lo register required -- `lsr r0,r8,#12'
[^:]*:64: Error: lo register required -- `lsr r8,r0'
[^:]*:64: Error: lo register required -- `lsr r0,r8'
[^:]*:65: Error: lo register required -- `ror r8,r0,#12'
[^:]*:65: Error: lo register required -- `ror r0,r8,#12'
[^:]*:65: Error: lo register required -- `ror r8,r0'
[^:]*:65: Error: lo register required -- `ror r0,r8'
[^:]*:66: Error: ror #imm not supported -- `ror r0,r1,#12'
[^:]*:69: Error: unshifted register required -- `add r0,r1,lsl#2'
[^:]*:70: Error: unshifted register required -- `add r0,r1,lsl r3'
[^:]*:71: Error: lo register required -- `add r8,r0,#1'
[^:]*:72: Error: lo register required -- `add r0,r8,#1'
[^:]*:73: Error: lo register required -- `add r8,#10'
[^:]*:74: Error: dest must overlap one source register -- `add r8,r1,r2'
[^:]*:75: Error: dest must overlap one source register -- `add r1,r8,r2'
[^:]*:76: Error: dest must overlap one source register -- `add r1,r2,r8'
[^:]*:77: Error: lo register required -- `add r8,pc,#4'
[^:]*:78: Error: lo register required -- `add r8,sp,#4'
[^:]*:80: Error: lo register required -- `sub r8,r0'
[^:]*:80: Error: lo register required -- `sub r0,r8'
[^:]*:80: Error: unshifted register required -- `sub r0,r1,lsl#2'
[^:]*:80: Error: unshifted register required -- `sub r0,r1,lsl r3'
[^:]*:81: Error: lo register required -- `sub r8,r0,#1'
[^:]*:82: Error: lo register required -- `sub r0,r8,#1'
[^:]*:83: Error: lo register required -- `sub r8,#10'
[^:]*:84: Error: lo register required -- `sub r8,r1,r2'
[^:]*:85: Error: lo register required -- `sub r1,r8,r2'
[^:]*:86: Error: lo register required -- `sub r1,r2,r8'
[^:]*:90: Error: only lo regs allowed with immediate -- `cmp r8,#255'
[^:]*:94: Error: only lo regs allowed with immediate -- `mov r8,#255'
[^:]*:106: Error: lo register required -- `ldr r8,\[r0\]'
[^:]*:106: Error: lo register required -- `ldr r0,\[r8\]'
[^:]*:106: Error: lo register required -- `ldr r0,\[r0,r8\]'
[^:]*:106: Error: Thumb does not support this addressing mode -- `ldr r0,\[r1,#4\]!'
[^:]*:106: Error: Thumb does not support this addressing mode -- `ldr r0,\[r1\],#4'
[^:]*:106: Error: Thumb does not support this addressing mode -- `ldr r0,\[r1,-r2\]'
[^:]*:106: Error: Thumb does not support this addressing mode -- `ldr r0,\[r1\],r2'
[^:]*:107: Error: lo register required -- `ldrb r8,\[r0\]'
[^:]*:107: Error: lo register required -- `ldrb r0,\[r8\]'
[^:]*:107: Error: lo register required -- `ldrb r0,\[r0,r8\]'
[^:]*:107: Error: Thumb does not support this addressing mode -- `ldrb r0,\[r1,#4\]!'
[^:]*:107: Error: Thumb does not support this addressing mode -- `ldrb r0,\[r1\],#4'
[^:]*:107: Error: Thumb does not support this addressing mode -- `ldrb r0,\[r1,-r2\]'
[^:]*:107: Error: Thumb does not support this addressing mode -- `ldrb r0,\[r1\],r2'
[^:]*:108: Error: lo register required -- `ldrh r8,\[r0\]'
[^:]*:108: Error: lo register required -- `ldrh r0,\[r8\]'
[^:]*:108: Error: lo register required -- `ldrh r0,\[r0,r8\]'
[^:]*:108: Error: Thumb does not support this addressing mode -- `ldrh r0,\[r1,#4\]!'
[^:]*:108: Error: Thumb does not support this addressing mode -- `ldrh r0,\[r1\],#4'
[^:]*:108: Error: Thumb does not support this addressing mode -- `ldrh r0,\[r1,-r2\]'
[^:]*:108: Error: Thumb does not support this addressing mode -- `ldrh r0,\[r1\],r2'
[^:]*:109: Error: lo register required -- `ldrsb r8,\[r0\]'
[^:]*:109: Error: lo register required -- `ldrsb r0,\[r8\]'
[^:]*:109: Error: lo register required -- `ldrsb r0,\[r0,r8\]'
[^:]*:109: Error: Thumb does not support this addressing mode -- `ldrsb r0,\[r1,#4\]!'
[^:]*:109: Error: Thumb does not support this addressing mode -- `ldrsb r0,\[r1\],#4'
[^:]*:109: Error: Thumb does not support this addressing mode -- `ldrsb r0,\[r1,-r2\]'
[^:]*:109: Error: Thumb does not support this addressing mode -- `ldrsb r0,\[r1\],r2'
[^:]*:110: Error: lo register required -- `ldrsh r8,\[r0\]'
[^:]*:110: Error: lo register required -- `ldrsh r0,\[r8\]'
[^:]*:110: Error: lo register required -- `ldrsh r0,\[r0,r8\]'
[^:]*:110: Error: Thumb does not support this addressing mode -- `ldrsh r0,\[r1,#4\]!'
[^:]*:110: Error: Thumb does not support this addressing mode -- `ldrsh r0,\[r1\],#4'
[^:]*:110: Error: Thumb does not support this addressing mode -- `ldrsh r0,\[r1,-r2\]'
[^:]*:110: Error: Thumb does not support this addressing mode -- `ldrsh r0,\[r1\],r2'
[^:]*:111: Error: lo register required -- `str r8,\[r0\]'
[^:]*:111: Error: lo register required -- `str r0,\[r8\]'
[^:]*:111: Error: lo register required -- `str r0,\[r0,r8\]'
[^:]*:111: Error: Thumb does not support this addressing mode -- `str r0,\[r1,#4\]!'
[^:]*:111: Error: Thumb does not support this addressing mode -- `str r0,\[r1\],#4'
[^:]*:111: Error: Thumb does not support this addressing mode -- `str r0,\[r1,-r2\]'
[^:]*:111: Error: Thumb does not support this addressing mode -- `str r0,\[r1\],r2'
[^:]*:112: Error: lo register required -- `strb r8,\[r0\]'
[^:]*:112: Error: lo register required -- `strb r0,\[r8\]'
[^:]*:112: Error: lo register required -- `strb r0,\[r0,r8\]'
[^:]*:112: Error: Thumb does not support this addressing mode -- `strb r0,\[r1,#4\]!'
[^:]*:112: Error: Thumb does not support this addressing mode -- `strb r0,\[r1\],#4'
[^:]*:112: Error: Thumb does not support this addressing mode -- `strb r0,\[r1,-r2\]'
[^:]*:112: Error: Thumb does not support this addressing mode -- `strb r0,\[r1\],r2'
[^:]*:113: Error: lo register required -- `strh r8,\[r0\]'
[^:]*:113: Error: lo register required -- `strh r0,\[r8\]'
[^:]*:113: Error: lo register required -- `strh r0,\[r0,r8\]'
[^:]*:113: Error: Thumb does not support this addressing mode -- `strh r0,\[r1,#4\]!'
[^:]*:113: Error: Thumb does not support this addressing mode -- `strh r0,\[r1\],#4'
[^:]*:113: Error: Thumb does not support this addressing mode -- `strh r0,\[r1,-r2\]'
[^:]*:113: Error: Thumb does not support this addressing mode -- `strh r0,\[r1\],r2'
[^:]*:115: Error: Thumb does not support this addressing mode -- `ldr r0,\[r1,r2,lsl#1\]'
[^:]*:116: Error: Thumb does not support this addressing mode -- `str r0,\[r1,r2,lsl#1\]'
[^:]*:119: Error: lo register required -- `ldmia r8!,{r1,r2}'
[^:]*:120: Error: lo register required -- `ldmia r7!,{r8}'
[^:]*:121: Warning: this instruction will write back the base register
[^:]*:122: Warning: this instruction will not write back the base register
[^:]*:124: Error: lo register required -- `stmia r8!,{r1,r2}'
[^:]*:125: Error: lo register required -- `stmia r7!,{r8}'
[^:]*:126: Warning: this instruction will write back the base register
[^:]*:127: Warning: value stored for r7 is UNPREDICTABLE
[^:]*:129: Error: invalid register list to push/pop instruction -- `push {r8,r9}'
[^:]*:130: Error: invalid register list to push/pop instruction -- `pop {r8,r9}'
[^:]*:133: Error: immediate value out of range -- `bkpt #257'
[^:]*:134: Error: Thumb does not support the 2-argument form of this instruction -- `cpsie ai,#5'
[^:]*:135: Error: Thumb does not support the 2-argument form of this instruction -- `cpsid ai,#5'
[^:]*:138: Error: Thumb does not support conditional execution
