.*: Assembler messages:
.*:5: Warning: Use of 'ld8.mov' violates RAW dependency .*number is 2
.*:4: Warning: This is the location of the conflicting usage
.*:8: Warning: Use of 'mov' violates RAW dependency .*number is 2
.*:7: Warning: This is the location of the conflicting usage
