
	.text

	fsqrtd	%fp0,%fp0
	fsqrtl	%fp@(8),%fp0
	fsqrts	%fp@(8),%fp0
	fsqrtw	%fp@(8),%fp0
	fsqrtd	%fp@(8),%fp0
	fsqrtb	%fp@(8),%fp0
	fssqrtd	%fp0,%fp0
	fssqrtl	%fp@(8),%fp0
	fssqrts	%fp@(8),%fp0
	fssqrtw	%fp@(8),%fp0
	fssqrtd	%fp@(8),%fp0
	fssqrtb	%fp@(8),%fp0
	fdsqrtd	%fp0,%fp0
	fdsqrtl	%fp@(8),%fp0
	fdsqrts	%fp@(8),%fp0
	fdsqrtw	%fp@(8),%fp0
	fdsqrtd	%fp@(8),%fp0
	fdsqrtb	%fp@(8),%fp0
	fabsd	%fp0,%fp0
	fabsl	%fp@(8),%fp0
	fabss	%fp@(8),%fp0
	fabsw	%fp@(8),%fp0
	fabsd	%fp@(8),%fp0
	fabsb	%fp@(8),%fp0
	fsabsd	%fp0,%fp0
	fsabsl	%fp@(8),%fp0
	fsabss	%fp@(8),%fp0
	fsabsw	%fp@(8),%fp0
	fsabsd	%fp@(8),%fp0
	fsabsb	%fp@(8),%fp0
	fdabsd	%fp0,%fp0
	fdabsl	%fp@(8),%fp0
	fdabss	%fp@(8),%fp0
	fdabsw	%fp@(8),%fp0
	fdabsd	%fp@(8),%fp0
	fdabsb	%fp@(8),%fp0
	fnegd	%fp0,%fp0
	fnegl	%fp@(8),%fp0
	fnegs	%fp@(8),%fp0
	fnegw	%fp@(8),%fp0
	fnegd	%fp@(8),%fp0
	fnegb	%fp@(8),%fp0
	fsnegd	%fp0,%fp0
	fsnegl	%fp@(8),%fp0
	fsnegs	%fp@(8),%fp0
	fsnegw	%fp@(8),%fp0
	fsnegd	%fp@(8),%fp0
	fsnegb	%fp@(8),%fp0
	fdnegd	%fp0,%fp0
	fdnegl	%fp@(8),%fp0
	fdnegs	%fp@(8),%fp0
	fdnegw	%fp@(8),%fp0
	fdnegd	%fp@(8),%fp0
	fdnegb	%fp@(8),%fp0
	fdivd	%fp0,%fp0
	fdivl	%fp@(8),%fp0
	fdivs	%fp@(8),%fp0
	fdivw	%fp@(8),%fp0
	fdivd	%fp@(8),%fp0
	fdivb	%fp@(8),%fp0
	fsdivd	%fp0,%fp0
	fsdivl	%fp@(8),%fp0
	fsdivs	%fp@(8),%fp0
	fsdivw	%fp@(8),%fp0
	fsdivd	%fp@(8),%fp0
	fsdivb	%fp@(8),%fp0
	fddivd	%fp0,%fp0
	fddivl	%fp@(8),%fp0
	fddivs	%fp@(8),%fp0
	fddivw	%fp@(8),%fp0
	fddivd	%fp@(8),%fp0
	fddivb	%fp@(8),%fp0
	faddd	%fp0,%fp0
	faddl	%fp@(8),%fp0
	fadds	%fp@(8),%fp0
	faddw	%fp@(8),%fp0
	faddd	%fp@(8),%fp0
	faddb	%fp@(8),%fp0
	fsaddd	%fp0,%fp0
	fsaddl	%fp@(8),%fp0
	fsadds	%fp@(8),%fp0
	fsaddw	%fp@(8),%fp0
	fsaddd	%fp@(8),%fp0
	fsaddb	%fp@(8),%fp0
	fdaddd	%fp0,%fp0
	fdaddl	%fp@(8),%fp0
	fdadds	%fp@(8),%fp0
	fdaddw	%fp@(8),%fp0
	fdaddd	%fp@(8),%fp0
	fdaddb	%fp@(8),%fp0
	fmuld	%fp0,%fp0
	fmull	%fp@(8),%fp0
	fmuls	%fp@(8),%fp0
	fmulw	%fp@(8),%fp0
	fmuld	%fp@(8),%fp0
	fmulb	%fp@(8),%fp0
	fsmuld	%fp0,%fp0
	fsmull	%fp@(8),%fp0
	fsmuls	%fp@(8),%fp0
	fsmulw	%fp@(8),%fp0
	fsmuld	%fp@(8),%fp0
	fsmulb	%fp@(8),%fp0
	fdmuld	%fp0,%fp0
	fdmull	%fp@(8),%fp0
	fdmuls	%fp@(8),%fp0
	fdmulw	%fp@(8),%fp0
	fdmuld	%fp@(8),%fp0
	fdmulb	%fp@(8),%fp0
	fsubd	%fp0,%fp0
	fsubl	%fp@(8),%fp0
	fsubs	%fp@(8),%fp0
	fsubw	%fp@(8),%fp0
	fsubd	%fp@(8),%fp0
	fsubb	%fp@(8),%fp0
	fssubd	%fp0,%fp0
	fssubl	%fp@(8),%fp0
	fssubs	%fp@(8),%fp0
	fssubw	%fp@(8),%fp0
	fssubd	%fp@(8),%fp0
	fssubb	%fp@(8),%fp0
	fdsubd	%fp0,%fp0
	fdsubl	%fp@(8),%fp0
	fdsubs	%fp@(8),%fp0
	fdsubw	%fp@(8),%fp0
	fdsubd	%fp@(8),%fp0
	fdsubb	%fp@(8),%fp0
	fmoved	%fp0,%fp0
	fmovel	%fp@(8),%fp0
	fmoves	%fp@(8),%fp0
	fmovew	%fp@(8),%fp0
	fmoved	%fp@(8),%fp0
	fmoveb	%fp@(8),%fp0
	fsmoved	%fp0,%fp0
	fsmovel	%fp@(8),%fp0
	fsmoves	%fp@(8),%fp0
	fsmovew	%fp@(8),%fp0
	fsmoved	%fp@(8),%fp0
	fsmoveb	%fp@(8),%fp0
	fdmoved	%fp0,%fp0
	fdmovel	%fp@(8),%fp0
	fdmoves	%fp@(8),%fp0
	fdmovew	%fp@(8),%fp0
	fdmoved	%fp@(8),%fp0
	fdmoveb	%fp@(8),%fp0
	fintd	%fp0,%fp0
	fintl	%fp@(8),%fp0
	fints	%fp@(8),%fp0
	fintw	%fp@(8),%fp0
	fintd	%fp@(8),%fp0
	fintb	%fp@(8),%fp0
	fintrzd	%fp0,%fp0
	fintrzl	%fp@(8),%fp0
	fintrzs	%fp@(8),%fp0
	fintrzw	%fp@(8),%fp0
	fintrzd	%fp@(8),%fp0
	fintrzb	%fp@(8),%fp0
	fcmpd	%fp0,%fp0
	fcmpl	%fp@(8),%fp0
	fcmps	%fp@(8),%fp0
	fcmpw	%fp@(8),%fp0
	fcmpd	%fp@(8),%fp0
	fcmpb	%fp@(8),%fp0
