.*: Assembler messages:
.*:39: Warning: DSP immediate not in range 0..7 \(4294967295\)
.*:42: Warning: DSP immediate not in range 0..7 \(8\)
.*:44: Warning: DSP immediate not in range 0..15 \(4294967295\)
.*:47: Warning: DSP immediate not in range 0..15 \(16\)
.*:49: Warning: DSP immediate not in range 0..15 \(4294967295\)
.*:52: Warning: DSP immediate not in range 0..15 \(16\)
.*:54: Warning: DSP immediate not in range 0..31 \(4294967295\)
.*:57: Warning: DSP immediate not in range 0..31 \(32\)
.*:59: Warning: DSP immediate not in range 0..7 \(4294967295\)
.*:62: Warning: DSP immediate not in range 0..7 \(8\)
.*:64: Warning: DSP immediate not in range 0..15 \(4294967295\)
.*:67: Warning: DSP immediate not in range 0..15 \(16\)
.*:69: Warning: DSP immediate not in range 0..15 \(4294967295\)
.*:72: Warning: DSP immediate not in range 0..15 \(16\)
.*:74: Warning: DSP immediate not in range 0..31 \(4294967295\)
.*:77: Warning: DSP immediate not in range 0..31 \(32\)
.*:99: Warning: DSP immediate not in range 0..255 \(4294967295\)
.*:102: Warning: DSP immediate not in range 0..255 \(256\)
.*:104: Warning: DSP immediate not in range -512..511 \(-513\)
.*:107: Warning: DSP immediate not in range -512..511 \(512\)
.*:121: Warning: DSP immediate not in range 0..31 \(4294967295\)
.*:124: Warning: DSP immediate not in range 0..31 \(32\)
.*:125: Warning: DSP immediate not in range 0..31 \(4294967295\)
.*:128: Warning: DSP immediate not in range 0..31 \(32\)
.*:129: Warning: DSP immediate not in range 0..31 \(4294967295\)
.*:132: Warning: DSP immediate not in range 0..31 \(32\)
.*:133: Warning: DSP immediate not in range 0..31 \(4294967295\)
.*:136: Warning: DSP immediate not in range 0..31 \(32\)
.*:141: Warning: DSP immediate not in range 0..31 \(4294967295\)
.*:144: Warning: DSP immediate not in range 0..31 \(32\)
.*:146: Warning: DSP immediate not in range 0..31 \(4294967295\)
.*:149: Warning: DSP immediate not in range 0..31 \(32\)
.*:151: Warning: DSP immediate not in range -32..31 \(-33\)
.*:154: Warning: DSP immediate not in range -32..31 \(32\)
.*:161: Warning: DSP immediate not in range 0..63 \(4294967295\)
.*:164: Warning: DSP immediate not in range 0..63 \(64\)
.*:166: Warning: DSP immediate not in range 0..63 \(4294967295\)
.*:169: Warning: DSP immediate not in range 0..63 \(64\)
