#source: b-twoinsn.s
#source: b-badfil2.s
#source: b-post1.s
#source: b-goodmain.s
#ld: --oformat binary
#objcopy_linked_file:
#error: invalid mmo file: file name for number 42 was not specified before use
