.*: Assembler messages:
.*:6: Error: bad expression
.*:6: Error: illegal operands `addiu'
.*:7: Error: bad expression
.*:7: Error: illegal operands `addiu'
.*:8: Error: bad expression
.*:8: Error: missing '\)'
.*:8: Error: illegal operands `addiu'
.*:9: Error: bad expression
.*:9: Error: missing '\)'
.*:9: Error: illegal operands `addiu'
