SCRIPT_NAME=elf
OUTPUT_FORMAT="elf32-am33lin"
TEXT_START_ADDR=0x8000000
MAXPAGESIZE=0x1000
NONPAGED_TEXT_START_ADDR=0x8000000
ARCH=mn10300
MACHINE=
NOP=0xcbcb
TEMPLATE_NAME=elf32
GENERATE_SHLIB_SCRIPT=yes

