#source: start.s
#source: jumpa.s
#source: pad2p26m32.s
#source: pad16.s
#source: pad4.s
#source: pad4.s
#source: a.s
#as: -no-expand
#ld: -m elf64mmix
#error: relocation truncated to fit: R_MMIX_ADDR27 against symbol `a'
