	.h8300s
	.text
h8300s_rotate_shift:
	rotl.b r0l
	rotl.b #2,r0l
	rotl.w r0
	rotl.w #2,r0
	rotl.l er0
	rotl.l #2,er0
	rotr.b r0l
	rotr.b #2,r0l
	rotr.w r0
	rotr.w #2,r0
	rotr.l er0
	rotr.l #2,er0
	rotxl.b r0l
	rotxl.b #2,r0l
	rotxl.w r0
	rotxl.w #2,r0
	rotxl.l er0
	rotxl.l #2,er0
	rotxr.b r0l
	rotxr.b #2,r0l
	rotxr.w r0
	rotxr.w #2,r0
	rotxr.l er0
	rotxr.l #2,er0
	shal.b r0l
	shal.b #2,r0l
	shal.w r0
	shal.w #2,r0
	shal.l er0
	shal.l #2,er0
	shar.b r0l
	shar.b #2,r0l
	shar.w r0
	shar.w #2,r0
	shar.l er0
	shar.l #2,er0
	shll.b r0l
	shll.b #2,r0l
	shll.w r0
	shll.w #2,r0
	shll.l er0
	shll.l #2,er0
	shlr.b r0l
	shlr.b #2,r0l
	shlr.w r0
	shlr.w #2,r0
	shlr.l er0
	shlr.l #2,er0
