[^:]*: Assembler messages:
[^:]*:5: Error: r15 not allowed here -- `mul r15,r1,r2'
[^:]*:6: Error: r15 not allowed here -- `mul r1,r15,r2'
[^:]*:7: Error: r15 not allowed here -- `mla r15,r2,r3,r4'
[^:]*:8: Error: r15 not allowed here -- `mla r1,r15,r3,r4'
[^:]*:9: Error: r15 not allowed here -- `mla r1,r2,r15,r4'
[^:]*:10: Error: r15 not allowed here -- `mla r1,r2,r3,r15'
[^:]*:11: Error: r15 not allowed here -- `smlabb r15,r2,r3,r4'
[^:]*:12: Error: r15 not allowed here -- `smlabb r1,r15,r3,r4'
[^:]*:13: Error: r15 not allowed here -- `smlabb r1,r2,r15,r4'
[^:]*:14: Error: r15 not allowed here -- `smlabb r1,r2,r3,r15'
[^:]*:15: Error: r15 not allowed here -- `smlalbb r15,r2,r3,r4'
[^:]*:16: Error: r15 not allowed here -- `smlalbb r1,r15,r3,r4'
[^:]*:17: Error: r15 not allowed here -- `smlalbb r1,r2,r15,r4'
[^:]*:18: Error: r15 not allowed here -- `smlalbb r1,r2,r3,r15'
[^:]*:19: Error: r15 not allowed here -- `smulbb r15,r2,r3'
[^:]*:20: Error: r15 not allowed here -- `smulbb r1,r15,r3'
[^:]*:21: Error: r15 not allowed here -- `smulbb r1,r2,r15'
[^:]*:22: Error: r15 not allowed here -- `qadd r15,r2,r3'
[^:]*:23: Error: r15 not allowed here -- `qadd r1,r15,r3'
[^:]*:24: Error: r15 not allowed here -- `qadd r1,r2,r15'
[^:]*:25: Error: r15 not allowed here -- `qadd16 r15,r2,r3'
[^:]*:26: Error: r15 not allowed here -- `qadd16 r1,r15,r3'
[^:]*:27: Error: r15 not allowed here -- `qadd16 r1,r2,r15'
[^:]*:28: Error: r15 not allowed here -- `clz r15,r2'
[^:]*:29: Error: r15 not allowed here -- `clz r1,r15'
[^:]*:30: Error: r15 not allowed here -- `umaal r15,r2,r3,r4'
[^:]*:31: Error: r15 not allowed here -- `umaal r1,r15,r3,r4'
[^:]*:32: Error: r15 not allowed here -- `umaal r1,r2,r15,r4'
[^:]*:33: Error: r15 not allowed here -- `umaal r1,r2,r3,r15'
[^:]*:34: Error: r15 not allowed here -- `strex r15,r2,[[]r3[]]'
[^:]*:35: Error: r15 not allowed here -- `strex r1,r15,[[]r3[]]'
[^:]*:36: Error: instruction does not accept this addressing mode -- `strex r1,r2,[[]r15[]]'
[^:]*:37: Error: r15 not allowed here -- `ssat r15,#1,r2'
[^:]*:38: Error: r15 not allowed here -- `ssat r1,#1,r15'
[^:]*:39: Error: r15 not allowed here -- `ssat16 r15,#1,r2'
[^:]*:40: Error: r15 not allowed here -- `ssat16 r1,#1,r15'
[^:]*:41: Error: r15 not allowed here -- `smmul r15,r2,r3'
[^:]*:42: Error: r15 not allowed here -- `smmul r1,r15,r3'
[^:]*:43: Error: r15 not allowed here -- `smmul r1,r2,r15'
[^:]*:44: Error: r15 not allowed here -- `smlald r15,r2,r3,r4'
[^:]*:45: Error: r15 not allowed here -- `smlald r1,r15,r3,r4'
[^:]*:46: Error: r15 not allowed here -- `smlald r1,r2,r15,r4'
[^:]*:47: Error: r15 not allowed here -- `smlald r1,r2,r3,r15'
[^:]*:48: Error: r15 not allowed here -- `smlad r15,r2,r3,r4'
[^:]*:49: Error: r15 not allowed here -- `smlad r1,r15,r3,r4'
[^:]*:50: Error: r15 not allowed here -- `smlad r1,r2,r15,r4'
[^:]*:51: Error: r15 not allowed here -- `smlad r1,r2,r3,r15'
[^:]*:52: Error: r15 not allowed here -- `sxth r15,r2'
[^:]*:53: Error: r15 not allowed here -- `sxth r1,r15'
[^:]*:54: Error: r15 not allowed here -- `sxtah r15,r2,r3'
[^:]*:55: Error: r15 not allowed here -- `sxtah r1,r15,r3'
[^:]*:56: Error: r15 not allowed here -- `sxtah r1,r2,r15'
[^:]*:57: Error: r15 not allowed here -- `rfeda r15'
[^:]*:58: Error: r15 not allowed here -- `rev r15,r2'
[^:]*:59: Error: r15 not allowed here -- `rev r1,r15'
[^:]*:60: Error: r15 not allowed here -- `pkhtb r15,r2,r3'
[^:]*:61: Error: r15 not allowed here -- `pkhtb r1,r15,r3'
[^:]*:62: Error: r15 not allowed here -- `pkhtb r1,r2,r15'
[^:]*:63: Error: r15 not allowed here -- `ldrex r15,[[]r2[]]'
[^:]*:64: Error: instruction does not accept this addressing mode -- `ldrex r1,[[]r15[]]'
[^:]*:65: Error: r15 not allowed here -- `swp r15,r2,[[]r3[]]'
[^:]*:66: Error: r15 not allowed here -- `swp r1,r15,[[]r3[]]'
[^:]*:67: Error: r15 not allowed here -- `swp r1,r2,[[]r15[]]'
