#source: start.s
#source: a.s
#source: pad2p18m32.s
#source: pad16.s
#source: pad4.s
#source: pad4.s
#source: pad4.s
#source: getaa.s
#as: -no-expand
#ld: -m mmo
#error: relocation truncated to fit: R_MMIX_ADDR19 against `a'$
