#readelf: -wl
#source: rd-pic-1.s
#as: --em=criself --pic --gdwarf2

# PIC prefix relocs.
#...
 Line Number Statements:
  Extended opcode 2: set Address to 0x0
  Special opcode .*: advance Address by 0 to 0x0 and Line by 5 to 6
  Special opcode .*: advance Address by 6 to 0x6 and Line by 1 to 7
  Special opcode .*: advance Address by 8 to 0xe and Line by 1 to 8
  Special opcode .*: advance Address by 6 to 0x14 and Line by 1 to 9
  Special opcode .*: advance Address by 6 to 0x1a and Line by 1 to 10
  Special opcode .*: advance Address by 6 to 0x20 and Line by 1 to 11
  Special opcode .*: advance Address by 8 to 0x28 and Line by 1 to 12
  Special opcode .*: advance Address by 6 to 0x2e and Line by 1 to 13
  Special opcode .*: advance Address by 4 to 0x32 and Line by 3 to 16
  Special opcode .*: advance Address by 8 to 0x3a and Line by 1 to 17
  Special opcode .*: advance Address by 6 to 0x40 and Line by 1 to 18
  Special opcode .*: advance Address by 6 to 0x46 and Line by 1 to 19
  Special opcode .*: advance Address by 8 to 0x4e and Line by 1 to 20
  Special opcode .*: advance Address by 8 to 0x56 and Line by 1 to 21
  Special opcode .*: advance Address by 8 to 0x5e and Line by 1 to 22
  Special opcode .*: advance Address by 8 to 0x66 and Line by 1 to 23
  Special opcode .*: advance Address by 8 to 0x6e and Line by 1 to 24
  Special opcode .*: advance Address by 8 to 0x76 and Line by 1 to 25
  Special opcode .*: advance Address by 6 to 0x7c and Line by 1 to 26
  Special opcode .*: advance Address by 6 to 0x82 and Line by 1 to 27
  Special opcode .*: advance Address by 6 to 0x88 and Line by 1 to 28
  Special opcode .*: advance Address by 6 to 0x8e and Line by 1 to 29
  Special opcode .*: advance Address by 6 to 0x94 and Line by 1 to 30
  Special opcode .*: advance Address by 6 to 0x9a and Line by 1 to 31
  Advance PC by 8 to 0xa2
  Extended opcode 1: End of Sequence
