.*: Assembler messages:
.*:12: Error: Improper position \([0-9]*\)
.*:15: Error: Improper position \(64\)
.*:18: Error: Improper extract size \(0, position 0\)
.*:21: Error: Improper extract size \(65, position 0\)
.*:31: Error: Improper extract size \(64, position 1\)
.*:33: Error: Improper extract size \(2, position 63\)
.*:34: Error: Improper extract size \(63, position 63\)
.*:35: Error: Improper extract size \(64, position 63\)
.*:40: Error: Improper position \([0-9]*\)
.*:43: Error: Improper position \(32\)
.*:46: Error: Improper extract size \(32, position 0\)
.*:49: Error: Improper extract size \(65, position 0\)
.*:59: Error: Improper extract size \(64, position 1\)
.*:61: Error: Improper extract size \(34, position 31\)
.*:62: Error: Improper extract size \(63, position 31\)
.*:63: Error: Improper extract size \(64, position 31\)
.*:68: Error: Improper position \(31\)
.*:71: Error: Improper position \(64\)
.*:74: Error: Improper extract size \(0, position 32\)
.*:77: Error: Improper extract size \(33, position 32\)
.*:87: Error: Improper extract size \(32, position 33\)
.*:89: Error: Improper extract size \(2, position 63\)
.*:90: Error: Improper extract size \(31, position 63\)
.*:91: Error: Improper extract size \(32, position 63\)
.*:96: Error: Improper position \([0-9]*\)
.*:99: Error: Improper position \(64\)
.*:102: Error: Improper insert size \(0, position 0\)
.*:105: Error: Improper insert size \(65, position 0\)
.*:115: Error: Improper insert size \(64, position 1\)
.*:117: Error: Improper insert size \(2, position 63\)
.*:118: Error: Improper insert size \(63, position 63\)
.*:119: Error: Improper insert size \(64, position 63\)
.*:124: Error: Improper position \([0-9]*\)
.*:127: Error: Improper position \(32\)
.*:130: Error: Improper insert size \(1, position 31\)
.*:133: Error: Improper insert size \(65, position 0\)
.*:136: Error: Improper insert size \(2, position 0\)
.*:137: Error: Improper insert size \(3, position 0\)
.*:140: Error: Improper insert size \(2, position 1\)
.*:141: Error: Improper insert size \(3, position 1\)
.*:143: Error: Improper insert size \(64, position 1\)
.*:144: Error: Improper insert size \(2, position 30\)
.*:146: Error: Improper insert size \(63, position 30\)
.*:147: Error: Improper insert size \(64, position 30\)
.*:150: Error: Improper insert size \(63, position 31\)
.*:151: Error: Improper insert size \(64, position 31\)
.*:156: Error: Improper position \(31\)
.*:159: Error: Improper position \(64\)
.*:162: Error: Improper insert size \(0, position 32\)
.*:165: Error: Improper insert size \(33, position 32\)
.*:175: Error: Improper insert size \(32, position 33\)
.*:178: Error: Improper insert size \(31, position 62\)
.*:179: Error: Improper insert size \(32, position 62\)
.*:181: Error: Improper insert size \(2, position 63\)
.*:182: Error: Improper insert size \(31, position 63\)
.*:183: Error: Improper insert size \(32, position 63\)
