.*: Assembler messages:
.*:8: Error: First operand to \.save\.g must be a positive 4-bit constant
.*:10: Error: First operand to \.save\.g must be a positive 4-bit constant
.*:12: Error: First operand to \.save\.g must be a positive 4-bit constant
.*:16: Warning: Previous .save incomplete
#FIXME .*:18: Error: Register r4 was already saved
.*:20: Error: Operand to \.save\.f must be a positive 20-bit constant
.*:22: Error: Operand to \.save\.f must be a positive 20-bit constant
.*:24: Error: Operand to \.save\.f must be a positive 20-bit constant
.*:28: Warning: Previous .save incomplete
#FIXME .*:30: Error: Register f2 was already saved
.*:32: Error: First operand to \.save\.b must be a positive 5-bit constant
.*:34: Error: First operand to \.save\.b must be a positive 5-bit constant
.*:36: Error: First operand to \.save\.b must be a positive 5-bit constant
.*:40: Warning: Previous .save incomplete
#FIXME .*:42: Error: Register b1 was already saved
.*:44: Error: Operand 2 to \.spillreg must be a writable register
.*:46: Error: Operand 1 to \.spillreg must be a preserved register
.*:48: Error: Operand 1 to \.spillreg must be a preserved register
.*:50: Error: Operand 1 to \.spillreg must be a preserved register
.*:52: Error: Operand 2 to \.spillreg must be a writable register
.*:54: Error: Operand 2 to \.spillreg must be a writable register
.*:56: Error: Operand 1 to \.spillreg must be a preserved register
#FIXME .*:58: Error: Floating point register cannot be spilled to general register
#FIXME .*:60: Error: Floating point register cannot be spilled to branch register
.*:62: Warning: Pointless use of p0 as first operand to \.spillreg\.p
.*:64: Error: Operand 3 to \.spillreg.p must be a writable register
.*:66: Error: Operand 3 to \.spillreg.p must be a writable register
.*:68: Warning: Pointless use of p0 as first operand to \.restorereg\.p
.*:78: Error: Operands to \.save\.gf may not be both zero
.*:80: Error: First operand to \.save\.gf must be a non-negative 4-bit constant
.*:82: Error: Second operand to \.save\.gf must be a non-negative 20-bit constant
.*:84: Error: First operand to \.save\.gf must be a non-negative 4-bit constant
.*:86: Error: Second operand to \.save\.gf must be a non-negative 20-bit constant
.*:90: Warning: Previous .save incomplete
#FIXME .*:92: Error: Register r4 was already saved
#FIXME .*:94: Error: Register f2 was already saved
.*:98: Error: Epilogue count of 2 exceeds number of nested prologues \(1\)
.*:100: Error: Missing \.label_state 2
.*:108: Error: First operand to \.save\.g must be a positive 4-bit constant
#FIXME .*:110: Error: Second operand to \.save\.g must be a writable general registers
.*:112: Error: Second operand to \.save\.g must be the first of 2 general registers
.*:115: Error: First operand to \.save\.b must be a positive 5-bit constant
#FIXME .*:117: Error: Second operand to \.save\.b must be a writable general registers
.*:119: Error: Second operand to \.save\.b must be the first of 2 general registers
.*:128: Error: First operand to \.prologue must be a positive 4-bit constant
.*:134: Warning: Pointless use of zero first operand to \.prologue
.*:140: Error: First operand to \.prologue must be a positive 4-bit constant
#FIXME .*:141: Error: Operand to \.vframe must be a writable general registers
#FIXME .*:147: Error: Second operand to \.prologue must be a writable general registers
.*:153: Error: Second operand to \.prologue must be the first of 2 general registers
