.*: Assembler messages:
.*:3: (Error|Warning): Invalid use of `r0' as output operand
.*:4: (Error|Warning): Invalid use of `r0' as base update address operand
.*:5: (Error|Warning): Invalid duplicate use of `r1'
.*:6: (Error|Warning): Invalid use of `r0' as base update address operand
.*:7: (Error|Warning): Invalid duplicate use of `p1'
.*:8: (Error|Warning): Invalid use of `f0' as output operand
.*:9: (Error|Warning): Invalid use of `f1' as output operand
.*:10: (Error|Warning): Invalid use of `f0' as output operand
.*:11: (Error|Warning): Invalid use of `f1' as output operand
.*:12: (Error|Warning): Invalid use of `f0' as output operand
.*:12: (Error|Warning): Invalid use of `f1' as output operand
.*:13: (Error|Warning): Invalid simultaneous use of `f2' and `f4'
.*:14: (Error|Warning): Dangerous simultaneous use of `f31' and `f32'
