SCRIPT_NAME=elf
OUTPUT_FORMAT="elf32-powerpc"
TEXT_BASE=0x00002000
DYN_TEXT_BASE=0x00400000
TEXT_START_ADDR="(DEFINED(_DYNAMIC) ? ${DYN_TEXT_BASE} : ${TEXT_BASE})"
MAXPAGESIZE=0x10000
NONPAGED_TEXT_START_ADDR=0x1000
ARCH=powerpc
MACHINE=
TEMPLATE_NAME=elf32
GENERATE_SHLIB_SCRIPT=yes
ELF_INTERPRETER_NAME=\"/usr/lib/ld.so.1\"

# Leave room of SIZEOF_HEADERS before text.
EMBEDDED=
# PLT on PowerPC is generated at run-time.
BSS_PLT=
OTHER_READWRITE_SECTIONS="
  .got1         ${RELOCATING-0} : { *(.got1) }
  .got2         ${RELOCATING-0} : { *(.got2) }"
