# flag test

	flag	r0

	flag	1
	flag	2
	flag	4
	flag	8
	flag	16
	flag	32
	flag	64
	flag	128

	flag	0x80000001

	flag.lt	r0

	flag.gt	1
	flag.gt	2
	flag.gt	4
	flag.gt	8
	flag.gt	16
	flag.gt	32
	flag.gt	64
	flag.gt	128

	flag.ge	0x80000001
