#include "windows.h"

LANGUAGE 0, 0

1 MENU MOVEABLE PURE DISCARDABLE
LANGUAGE LANG_GERMAN, SUBLANG_GERMAN
BEGIN
	POPUP "&Datei" { MENUITEM "foo", 1 }
END

1 MENU MOVEABLE PURE DISCARDABLE
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
BEGIN
	POPUP "&File" { MENUITEM "foo", 1 }
END
