.explicit
_start:
{.mfi
	mov	ar0 = r0
	mov	ar127 = r0
} ;; {.mfi
	mov	ar47 = r0
	mov	ar112 = r0
} ;; {.mfi
	mov	ar48 = r0
	mov	ar111 = r0
} ;; {.mfi
	mov	ar63 = r0
	mov	ar64 = r0
} ;; {.mfi
	mov	ar112 = r0
	mov	ar63 = r0
} ;; {.mfi
	mov	ar127 = r0
	mov	ar48 = r0
} ;;
