#source: b-twoinsn.s
#source: b-post1.s
#source: b-badm2.s
#ld: --oformat binary
#objcopy_linked_file:
#error: invalid mmo file: YZ of lop_end .* not equal to the number of tetras to the preceding lop_stab
