GAS for MMIX .*/unsave-op.s 			page 1


   1              	#.*
   2 0000 FB00002D 	Main UNSAVE 0,\$45
   3 0004 FB00001F 	 UNSAVE 0,X
   4 0008 FB000000 	 UNSAVE 0,X0
   5              	X IS \$31
   6              	X0 IS \$0
GAS for MMIX .*/unsave-op\.s 			page 2


DEFINED SYMBOLS
.*/unsave-op.s:2      \.text:0000000000000000 Main
                            \*REG\*:000000000000001f X
                            \*REG\*:0000000000000000 X0

NO UNDEFINED SYMBOLS
