#source: b-twoinsn.s
#source: b-badfil1.s
#source: b-post1.s
#source: b-goodmain.s
#ld: --oformat binary
#objcopy_linked_file:
#error: invalid mmo file: file number 42 `bar.s', was already entered as `foo.s'
