/* This is the location of the online document giving instructions for
   reporting bugs.  If you distribute a modified version of binutils,
   please override this to refer to a document giving instructions for
   reporting bugs to you, not us.  (You are of course welcome to
   forward us bugs reported to you, if you determine that they are
   not bugs in your modifications.)  */

#ifndef REPORT_BUGS_TO
#define REPORT_BUGS_TO	"<URL:http://www.sourceware.org/bugzilla/>"
#endif
