.*: Assembler messages:
.*:9: Warning: relaxed out-of-range branch into a jump
.*:13: Warning: relaxed out-of-range branch into a jump
.*:18: Warning: relaxed out-of-range branch into a jump
.*:22: Warning: relaxed out-of-range branch into a jump
.*:27: Warning: relaxed out-of-range branch into a jump
.*:31: Warning: relaxed out-of-range branch into a jump
.*:36: Warning: relaxed out-of-range branch into a jump
.*:40: Warning: relaxed out-of-range branch into a jump
.*:45: Warning: relaxed out-of-range branch into a jump
